@@ -38,6 +38,8 @@ cb_rstto_zoom_out(GtkToolItem *item, RsttoPictureViewer *viewer);
 
 static void
 cb_rstto_fullscreen(GtkWidget *, GdkEventWindowState *event, RsttoPictureViewer *viewer);
+static gboolean
+cb_rstto_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event);
 static void
 cb_rstto_toggle_play(GtkImageMenuItem *item, RsttoNavigator *navigator);
 
@@ -69,6 +71,9 @@ cb_rstto_show_tv_h(GtkWidget *widget, RsttoThumbnailViewer *viewer);
 static void
 cb_rstto_hide_tv(GtkWidget *widget, RsttoThumbnailViewer *viewer);
 
+static void
+cb_rstto_toggle_toolbar(GtkWidget *widget, RsttoThumbnailViewer *viewer);
+
 static void
 cb_rstto_rotate_cw(GtkWidget *widget, RsttoNavigator *navigator);
 static void
@@ -78,13 +83,12 @@ cb_rstto_flip_h(GtkWidget *widget, RsttoNavigator *navigator);
 static void
 cb_rstto_flip_v(GtkWidget *widget, RsttoNavigator *navigator);
 
-static void
+static gboolean
 cb_rstto_key_press_event(GtkWidget *widget, GdkEventKey *event, RsttoNavigator *navigator);
 static void
 cb_rstto_nav_file_changed(RsttoNavigator *navigator, gint nr, RsttoNavigatorEntry *entry, GtkWindow *window);
 
 static gboolean window_fullscreen = FALSE;
-static gboolean viewer_scale = 1.0;
 static GtkWidget *menu_bar;
 static GtkWidget *app_tool_bar;
 static GtkWidget *status_bar;
@@ -98,6 +102,7 @@ static GtkWidget *thumbnail_viewer;
 static GtkRecentManager *recent_manager;
 static XfceRc *xfce_rc;
 static const gchar *thumbnail_viewer_orientation;
+static gint window_save_geometry_timer_id = 0;
 
 int main(int argc, char **argv)
 {
@@ -123,6 +128,9 @@ int main(int argc, char **argv)
     
     thumbnail_viewer_orientation = xfce_rc_read_entry(xfce_rc, "ThumbnailViewerOrientation", "horizontal");
     gboolean toolbar_open_dir = xfce_rc_read_bool_entry(xfce_rc, "ToolbarOpenDir", FALSE);
+    gboolean show_toolbar = xfce_rc_read_bool_entry(xfce_rc, "ShowToolBar", TRUE);
+    gint window_width = xfce_rc_read_int_entry(xfce_rc, "LastWindowWidth", 400);
+    gint window_height = xfce_rc_read_int_entry(xfce_rc, "LastWindowHeight", 300);
 
     GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     GtkAccelGroup *accel_group = gtk_accel_group_new();
@@ -231,12 +239,16 @@ int main(int argc, char **argv)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu_edit), menu_item_flip_h);
 
     GtkWidget *menu_item_view = gtk_menu_item_new_with_mnemonic(_("_View"));
+    GtkWidget *menu_item_toggle_toolbar = gtk_check_menu_item_new_with_mnemonic(_("Show Toolbar"));
     GtkWidget *menu_item_tv = gtk_menu_item_new_with_mnemonic(_("Thumbnail Viewer"));
     GtkWidget *menu_item_view_fs = gtk_image_menu_item_new_from_stock(GTK_STOCK_FULLSCREEN, NULL);
     gtk_widget_add_accelerator(menu_item_view_fs, "activate", accel_group, GDK_F11, 0,GTK_ACCEL_VISIBLE);
 
+    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item_toggle_toolbar), show_toolbar);
+
     GtkWidget *menu_view = gtk_menu_new();
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item_view), menu_view);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu_view), menu_item_toggle_toolbar);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu_view), menu_item_tv);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu_view), menu_item_view_fs);
 
@@ -302,12 +314,13 @@ int main(int argc, char **argv)
     GtkToolItem *spacer = gtk_tool_item_new();
     GtkToolItem *separator = gtk_separator_tool_item_new();
 
+    gtk_tool_item_set_is_important(previous, TRUE);
+    gtk_tool_item_set_is_important(forward, TRUE);
 
     gtk_tool_item_set_expand(spacer, TRUE);
     gtk_tool_item_set_homogeneous(spacer, FALSE);
 
-    gtk_widget_set_size_request(window, 400, 300);
-
+    gtk_window_set_default_size(GTK_WINDOW(window), window_width, window_height);
 
     gtk_container_add(GTK_CONTAINER(s_window), viewer);
     gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox1, TRUE, TRUE, 0);
@@ -372,6 +385,8 @@ int main(int argc, char **argv)
     g_signal_connect(G_OBJECT(menu_item_flip_v), "activate", G_CALLBACK(cb_rstto_flip_v), navigator);
     g_signal_connect(G_OBJECT(menu_item_flip_h), "activate", G_CALLBACK(cb_rstto_flip_h), navigator);
 
+    g_signal_connect(G_OBJECT(menu_item_toggle_toolbar), "activate", G_CALLBACK(cb_rstto_toggle_toolbar), thumbnail_viewer);
+
     g_signal_connect(G_OBJECT(menu_item_vtv), "activate", G_CALLBACK(cb_rstto_show_tv_v), thumbnail_viewer);
     g_signal_connect(G_OBJECT(menu_item_htv), "activate", G_CALLBACK(cb_rstto_show_tv_h), thumbnail_viewer);
     g_signal_connect(G_OBJECT(menu_item_ntv), "activate", G_CALLBACK(cb_rstto_hide_tv), thumbnail_viewer);
@@ -381,6 +396,7 @@ int main(int argc, char **argv)
     g_signal_connect(G_OBJECT(menu_item_view_fs), "activate", G_CALLBACK(cb_rstto_toggle_fullscreen), window);
 
     g_signal_connect(G_OBJECT(window), "window-state-event", G_CALLBACK(cb_rstto_fullscreen), viewer);
+    g_signal_connect(G_OBJECT(window), "configure-event", G_CALLBACK(cb_rstto_main_window_configure_event), NULL);
 
     gtk_container_add(GTK_CONTAINER(window), main_vbox);
 
@@ -393,6 +409,10 @@ int main(int argc, char **argv)
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item_ntv), TRUE);
         gtk_widget_hide(GTK_WIDGET(thumbnail_viewer));   
     }
+    if (show_toolbar == FALSE)
+    {
+        gtk_widget_hide(GTK_WIDGET(app_tool_bar));   
+    }
 
 
     gtk_main();
@@ -512,8 +532,6 @@ cb_rstto_fullscreen(GtkWidget *widget, GdkEventWindowState *event, RsttoPictureV
             gtk_widget_hide(menu_bar);
             gtk_widget_hide(app_tool_bar);
             gtk_widget_hide(status_bar);
-            viewer_scale = rstto_picture_viewer_get_scale(viewer);
-            rstto_picture_viewer_fit_scale(viewer);
         }
         else
         {
@@ -521,8 +539,6 @@ cb_rstto_fullscreen(GtkWidget *widget, GdkEventWindowState *event, RsttoPictureV
             gtk_widget_show(menu_bar);
             gtk_widget_show(app_tool_bar);
             gtk_widget_show(status_bar);
-            if(viewer_scale)
-                rstto_picture_viewer_set_scale(viewer, viewer_scale);
         }
     }
 }
@@ -647,7 +663,7 @@ cb_rstto_rotate_ccw(GtkWidget *widget, RsttoNavigator *navigator)
     //rstto_navigator_set_entry_rotation(navigator, entry, rotation);
 }
 
-static void
+static gboolean
 cb_rstto_key_press_event(GtkWidget *widget, GdkEventKey *event, RsttoNavigator *navigator)
 {
     GtkWindow *window = GTK_WINDOW(widget);
@@ -677,6 +693,7 @@ cb_rstto_key_press_event(GtkWidget *widget, GdkEventKey *event, RsttoNavigator *
                 break;
         }
     }
+    return FALSE;
 }
 
 static void
@@ -828,3 +845,75 @@ cb_rstto_open_recent(GtkRecentChooser *chooser, RsttoNavigator *navigator)
         thunar_vfs_path_unref(path);
     }
 }
+
+static void
+cb_rstto_toggle_toolbar(GtkWidget *widget, RsttoThumbnailViewer *viewer)
+{
+    gboolean active = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(widget));
+    if (active == TRUE)
+    {
+        gtk_widget_show(app_tool_bar);
+        xfce_rc_write_bool_entry(xfce_rc, "ShowToolBar", TRUE);
+    }
+    else
+    {
+        gtk_widget_hide(app_tool_bar);
+        xfce_rc_write_bool_entry(xfce_rc, "ShowToolBar", FALSE);
+    }
+}
+
+static gboolean
+rstto_window_save_geometry_timer (gpointer user_data)
+{
+    GtkWindow *window = GTK_WINDOW(user_data);
+    gint width = 0;
+    gint height = 0;
+    /* check if the window is still visible */
+    if (GTK_WIDGET_VISIBLE (window))
+    {
+        /* determine the current state of the window */
+        gint state = gdk_window_get_state (GTK_WIDGET (window)->window);
+
+        /* don't save geometry for maximized or fullscreen windows */
+        if ((state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) == 0)
+        {
+            /* determine the current width/height of the window... */
+            gtk_window_get_size (GTK_WINDOW (window), &width, &height);
+
+            /* ...and remember them as default for new windows */
+            xfce_rc_write_int_entry (xfce_rc, "LastWindowWidth", width);
+            xfce_rc_write_int_entry (xfce_rc, "LastWindowHeight", height);
+        }
+    }
+    return FALSE;
+}
+
+static void
+rstto_window_save_geometry_timer_destroy(gpointer user_data)
+{
+    window_save_geometry_timer_id = 0;
+}
+
+static gboolean
+cb_rstto_main_window_configure_event (GtkWidget *widget, GdkEventConfigure *event)
+{
+    /* shamelessly copied from thunar, written by benny */
+    /* check if we have a new dimension here */
+    if (widget->allocation.width != event->width || widget->allocation.height != event->height)
+    {
+        /* drop any previous timer source */
+        if (window_save_geometry_timer_id > 0)
+            g_source_remove (window_save_geometry_timer_id);
+
+        /* check if we should schedule another save timer */
+        if (GTK_WIDGET_VISIBLE (widget))
+        {
+            /* save the geometry one second after the last configure event */
+            window_save_geometry_timer_id = g_timeout_add_full (G_PRIORITY_LOW, 1000, rstto_window_save_geometry_timer,
+                widget, rstto_window_save_geometry_timer_destroy);
+        }
+    }
+
+    /* let Gtk+ handle the configure event */
+    return FALSE;
+}
