@@ -30,6 +30,8 @@ struct _RsttoPictureViewerPriv
     RsttoNavigator   *navigator;
     void             (*cb_value_changed)(GtkAdjustment *, RsttoPictureViewer *);
     gboolean          show_border;
+    GTimer           *timer;
+    gint              timeout_id;
 };
 
 static void
@@ -54,6 +56,9 @@ static gboolean
 rstto_picture_viewer_refresh(RsttoPictureViewer *viewer);
 
 static void
+rstto_picture_viewer_update(RsttoPictureViewer *viewer);
+
+static gboolean
 rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *, GtkAdjustment *, GtkAdjustment *);
 
 static void
@@ -62,6 +67,8 @@ static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
 static void
 cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *, GdkEventScroll *);
+static gboolean
+cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer);
 
 
 static GtkWidgetClass *parent_class = NULL;
@@ -103,6 +110,7 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
     gtk_widget_set_redraw_on_allocate(GTK_WIDGET(viewer), TRUE);
 
     viewer->priv->show_border = TRUE;
+    viewer->priv->timer = g_timer_new();
 
     g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
 }
@@ -324,7 +332,7 @@ rstto_picture_viewer_destroy(GtkObject *object)
 
 }
 
-static void
+static gboolean  
 rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *viewer, GtkAdjustment *hadjustment, GtkAdjustment *vadjustment)
 {
     if(viewer->hadjustment)
@@ -351,6 +359,7 @@ rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *viewer, GtkAdjus
         g_signal_connect(G_OBJECT(viewer->vadjustment), "value-changed", (GCallback)viewer->priv->cb_value_changed, viewer);
         g_object_ref(viewer->vadjustment);
     }
+    return TRUE;
 }
 
 static void
@@ -607,9 +616,31 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
     return changed;
 }
 
+static gboolean
+cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer)
+{
+    if (g_timer_elapsed(viewer->priv->timer, NULL) > 0.2)
+    {
+        g_timer_stop(viewer->priv->timer);
+        viewer->priv->timeout_id = 0;
+        rstto_picture_viewer_update(viewer);
+        return FALSE;
+    }
+    return TRUE;
+}
+
 static void
 cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer)
 {
+    g_timer_start(viewer->priv->timer);
+    if (viewer->priv->timeout_id == 0)
+        viewer->priv->timeout_id = g_timeout_add(200, (GSourceFunc)cb_rstto_picture_viewer_update_image, viewer);
+}
+
+static void
+rstto_picture_viewer_update(RsttoPictureViewer *viewer)
+{
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
     GtkWidget *widget = GTK_WIDGET(viewer);
     if(GTK_WIDGET_REALIZED(widget))
     {
@@ -655,13 +686,16 @@ cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *viewer, GdkEventScroll
             break;
         case GDK_SCROLL_DOWN:
         case GDK_SCROLL_RIGHT:
+            viewer->vadjustment->value = ((viewer->vadjustment->value + event->y) - viewer->vadjustment->page_size/2) * 1.2;
+            viewer->hadjustment->value = ((viewer->hadjustment->value + event->x) - viewer->hadjustment->page_size/2) * 1.2;
+
             rstto_navigator_entry_set_scale(entry, scale * 1.2);
             rstto_navigator_entry_set_fit_to_screen (entry, FALSE);
             break;
     }
+
     if(rstto_picture_viewer_refresh(viewer))
     {
         rstto_picture_viewer_paint(GTK_WIDGET(viewer));
     }
-
 }
